from django.db import models
from django.utils.translation import gettext_lazy as _

from huscy.project_design.models import DataAcquisitionMethod


def get_upload_path(instance, filename):
    project_id = instance.data_acquisition_method.session.experiment.project.id
    return f'projects/{project_id}/data_acquisition_methods/questionaires/{filename}'


class Questionaire(models.Model):
    data_acquisition_method = models.ForeignKey(DataAcquisitionMethod, on_delete=models.CASCADE)

    name = models.CharField(_('Name'), max_length=255)

    filehandle = models.FileField(upload_to=get_upload_path, verbose_name=_('Filehandle'))

    uploaded_at = models.DateTimeField(auto_now_add=True, editable=False)
    uploaded_by = models.CharField(max_length=255, editable=False)



from django.urls import include, path

from . import views
from huscy.project_design.urls import session_router


session_router.register('questionaires', views.QuestionaireViewSet, basename='questionaire')


urlpatterns = (
    path('api/', include(session_router.urls)),
)



from django.shortcuts import get_object_or_404
from rest_framework.mixins import CreateModelMixin, DestroyModelMixin, UpdateModelMixin
from rest_framework.permissions import IsAuthenticated
from rest_framework.viewsets import GenericViewSet

from . import serializer, services
from huscy.project_design.models import Session


class QuestionaireViewSet(CreateModelMixin, DestroyModelMixin, UpdateModelMixin, GenericViewSet):
    permission_classes = IsAuthenticated,
    serializer_class = serializer.QuestionaireSerializer

    def initial(self, request, *args, **kwargs):
        self.session = get_object_or_404(
            Session,
            experiment=self.kwargs['experiment_pk'],
            experiment__project=self.kwargs['project_pk'],
            pk=self.kwargs['session_pk'],
        )
        return super().initial(request, *args, **kwargs)

    def get_queryset(self):
        return services.get_questionaires(self.session)

    def perform_create(self, serializer):
        serializer.save(session=self.session)

    def perform_destroy(self, questionaire):
        services.delete_questionaire(questionaire)



from rest_framework import serializers

from . import models, services


class QuestionaireSerializer(serializers.ModelSerializer):
    uploaded_by = serializers.HiddenField(default=serializers.CurrentUserDefault())

    class Meta:
        model = models.Questionaire
        fields = (
            'id',
            'filehandle',
            'name',
            'uploaded_at',
            'uploaded_by',
        )
        read_only_fields = 'id', 'uploaded_at', 'uploaded_by'

    def get_fields(self):
        fields = super().get_fields()
        request = self.context.get('request')
        if request.method == 'PUT':
            fields['filehandle'].read_only = True
            fields['uploaded_by'].read_only = True
        return fields

    def create(self, validated_data):
        user = validated_data.pop('uploaded_by')
        return services.create_questionaire(user=user, **validated_data)

    def update(self, questionaire, validated_data):
        return services.update_questionaire(questionaire, **validated_data)



import warnings

from django.apps import AppConfig
from django.db import connection


class HuscyApp(AppConfig):
    default_auto_field = 'django.db.models.BigAutoField'
    name = 'huscy.data_acquisition_methods.questionaire'

    class HuscyAppMeta:
        pass

    def ready(self):
        from huscy.project_design.models import DataAcquisitionMethodType

        if 'project_design_dataacquisitionmethodtype' not in connection.introspection.table_names():
            warnings.warn('The migration files of the huscy.project_design app have not yet '
                          'been executed.')
            return

        DataAcquisitionMethodType.objects.get_or_create(
            short_name='questionaire',
            defaults=dict(
                name='Questionaire',
            ),
        )



# Generated by Django 4.2.11 on 2024-03-25 12:18

from django.db import migrations, models
import django.db.models.deletion
import huscy.data_acquisition_methods.questionaire.models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
        ('project_design', '0001_initial'),
    ]

    operations = [
        migrations.CreateModel(
            name='Questionaire',
            fields=[
                ('id', models.BigAutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=255, verbose_name='Name')),
                ('filehandle', models.FileField(upload_to=huscy.data_acquisition_methods.questionaire.models.get_upload_path, verbose_name='Filehandle')),
                ('uploaded_at', models.DateTimeField(auto_now_add=True)),
                ('uploaded_by', models.CharField(editable=False, max_length=255)),
                ('data_acquisition_method', models.ForeignKey(on_delete=django.db.models.deletion.CASCADE, to='project_design.dataacquisitionmethod')),
            ],
        ),
    ]



from django.shortcuts import get_object_or_404

from .models import Questionaire
from huscy.project_design.models import DataAcquisitionMethodType
from huscy.project_design.services import create_data_acquisition_method


def create_questionaire(session, name, filehandle, user, location=''):
    data_acquisition_method_type = get_object_or_404(DataAcquisitionMethodType, pk='questionaire')

    data_acquisition_method = create_data_acquisition_method(
        session=session,
        type=data_acquisition_method_type,
        location=location,
    )

    return Questionaire.objects.create(
        data_acquisition_method=data_acquisition_method,
        filehandle=filehandle,
        name=name,
        uploaded_by=user.get_full_name(),
    )


def delete_questionaire(questionaire):
    questionaire.data_acquisition_method.delete()


def get_questionaires(session):
    return (Questionaire.objects.filter(data_acquisition_method__session=session)
                                .order_by('data_acquisition_method__order'))


def update_questionaire(questionaire, name):
    questionaire.name = name
    questionaire.save()
    return questionaire



from os import path

from setuptools import find_namespace_packages, setup

from huscy.data_acquisition_methods.questionaire import __version__


with open(path.join(path.abspath(path.dirname(__file__)), 'README.md'), encoding='utf-8') as f:
    long_description = f.read()


setup(
    name='huscy.data-acquisition-methods.questionaire',
    version=__version__,
    license='AGPLv3+',

    description='',
    long_description=long_description,
    long_description_content_type='text/markdown',

    author='Stefan Bunde',
    author_email='stefanbunde+git@posteo.de',

    url='https://bitbucket.org/huscy/data_acquisition_methods.questionaire',

    packages=find_namespace_packages(include=['huscy.*']),

    install_requires=[
        'huscy.project_design',
    ],
    extras_require={
        'development': ['psycopg2-binary'],
        'testing': ['tox', 'watchdog==0.9'],
    },

    classifiers=[
        'Development Status :: 3 - Alpha',
        'License :: OSI Approved :: GNU Affero General Public License v3 or later (AGPLv3+)',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3',
        'Programming Language :: Python :: 3 :: Only',
        'Programming Language :: Python :: 3.8',
        'Programming Language :: Python :: 3.9',
        'Programming Language :: Python :: 3.10',
        'Programming Language :: Python :: 3.11',
        'Framework :: Django',
        'Framework :: Django :: 4.2',
        'Framework :: Django :: 5.0',
    ],
)
